//
// SWIG typemaps for std::string types
// Luigi Ballabio
// Apr 8, 2002
//
// Support for mzscheme 30x added 
// Hans Oesterholt
// hdnews -at- gawab -dot- com
// 2005-03-28
//
// MzScheme implementation

// ------------------------------------------------------------------------
// std::string is typemapped by value
// This can prevent exporting methods which return a string
// in order for the user to modify it.
// However, I think I'll wait until someone asks for it...
// ------------------------------------------------------------------------

%include exception.i

%{
#include <string>
%}

#ifndef SCHEME_STR_VAL 
/* 
 * This is MZSCHEME 299.100 or higher (30x).
 * From version 299.100 of mzscheme upwards,
 * strings are in unicode. These functions convert
 * to and from utf8 encodings of these strings.
 * NB! strlen(s) will be the size in bytes of
 * the string, not the actual length.
 */
#define SCHEME_STR_VAL(obj)  	SCHEME_BYTE_STR_VAL(scheme_char_string_to_byte_string(obj))
#define SCHEME_STRINGP(obj)     SCHEME_CHAR_STRINGP(obj)
#define scheme_make_string(s)   scheme_make_utf8_string(s)
#endif


namespace std {

    class string;

    /* Overloading check */

    %typemap(typecheck) string = char *;
    %typemap(typecheck) const string & = char *;

    %typemap(in) string {
        if (SCHEME_STRINGP($input))
            $1 = std::string(SCHEME_STR_VAL($input));
        else
            SWIG_exception(SWIG_TypeError, "string expected");
    }

    %typemap(in) const string & (std::string temp) {
        if (SCHEME_STRINGP($input)) {
            temp = std::string(SCHEME_STR_VAL($input));
            $1 = &temp;
        } else {
            SWIG_exception(SWIG_TypeError, "string expected");
        }
    }

    %typemap(out) string {
        $result = scheme_make_string($1.c_str());
    }

    %typemap(out) const string & {
        $result = scheme_make_string($1->c_str());
    }

}


